#include "predef.h" 
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <sim5272.h>
#include <cfinter.h>
#include <string.h>

extern "C" { 
void UserMain(void * pd);
extern const unsigned long CPU_CLOCK;
}

const char * AppName="bare_serial";



volatile uartstruct *BareUart;




/* Receiveing circular buffer stuff*/
OS_SEM SemDataRx;
char RxBuffer[1000];
volatile int char_put_index;
volatile int char_get_index;


/* Tranmitting buffer stuff.... */
const char *TxBuffer;
int TxLen;
OS_SEM SemDataTx;




INTERRUPT( BARE_ISR, 0x2400 )
{
while (BareUart->usr & 1)
	{//Process incomming chars */
	 
	 char c = ( BareUart->udb ); 
 

	 /* For this example we will store all the chars up to a \n in a circular buffer */
	 if (c!='\n')
	 {
		 
	 RxBuffer[char_put_index++]=c;
	 if (char_put_index==1000) char_put_index=0;
	 
	 if (char_put_index==char_get_index)
		{/* If we got here we overflowed our circular buffer and should handle the error */

		 char_get_index++;
		 if (char_get_index==1000) char_get_index=0;
		}
	 }
	 else
		 OSSemPost( &SemDataRx ); 
	}
if (BareUart->uisr & 1==0) return; /* Don't do TX if we shouldn't be */
while (BareUart->usr & 4)
	{
	 if (TxLen)
		{
		 TxLen--;
		 BareUart->udb=*(TxBuffer++);
	    }
   		else
		{//TxLen got to zero so we need to turn off the ISR stuff
		 BareUart->uisr=2; /* Only do RX interrupts */
		 BareUart->ucr = 0x08; /* TX disable */
		 OSSemPost(&SemDataTx);
		}
	}


}




void OpenBareSerial(int port, unsigned long baud)
{

OSSemInit(&SemDataRx,0);
OSSemInit(&SemDataTx,0); 


BareUart = &sim.uarts[port];

unsigned char ub1 = 0;
unsigned char ub2 = 0x41;


   switch ( baud )
   {
     case 115200:
       ub2 = 17;
       break;
     case 57600:
       ub2 = 34;
       break;
     case 38400:
       ub2 = 51;
       break;
     case 28800:
       ub2 = 68;
       break;
     case 19200:
       ub2 = 102;
       break;
     case 9600:
       ub2 = 203;
       break;
     case 4800:
       ub1 = 1;
       ub2 = 151;
       break;
     case 2400:
       ub1 = 3;
       ub2 = 46;
       break;
     case 1200:
       ub1 = 6;
       ub2 = 92;
       break;
     case 600:
       ub1 = 12;
       ub2 = 183;
       break;
     case 300:
       ub1 = 25;
       ub2 = 110;
       break;
     default:
       ub2 = 17;
   }


BareUart->ucr =  0x2a; /*Reset Rx Transmit disable Rx disable */
BareUart->ucr =  0x3a;    /*Reset Tx Transmit disable Rx disable */
BareUart->ucr =  0x1a;/*Transmit disable Rx disable reset mode reg */

BareUart->ucr =  0x10;
BareUart->umr =  0x13; /* umr1 8 data bits no parity */
BareUart->umr =  0x27;  /* umr2 1 stop*/ 

BareUart->usr =  0xDD; /* Use the internal timer for Baud Rate Generation */
BareUart->uisr =   0x02;  /*Enable Rx  interrupts */

BareUart->ubg1 = ub1; /* Actualy set the baud rate we calculated */
BareUart->ubg2 = ub2;


switch (port) /* Set up the hardware ports */
{
	case 0:
		vector_base.table[73 + port] = ( unsigned long ) &BARE_ISR;
		sim.icr2 &= ~( 0xF << 28 );
		sim.icr2 |= ( ( 0x8 | SERIAL_IRQ_LEVEL ) << 28 ) + 0;

    break;
case 1:
	vector_base.table[73 + port] = ( unsigned long ) &BARE_ISR;
	sim.icr2 &= ~( 0xF << 24 );
	sim.icr2 |= ( ( 0x8 | SERIAL_IRQ_LEVEL ) << 24 ) + 0;

	sim.pdcnt &= 0xFEAB;   // 1111 1110 1010 1011
	sim.pdcnt |= 0x02A8;   // 0000 0010 1010 1000
	break;
}//End of switch



BareUart->ucr = 0x05;/*Enable Tx and Rx */ 
}






/* This write will block until all the data is in the uart */
void IsrDrivenWrite(int port, const char * buffer, int len)
{
TxBuffer=buffer;
if(len==0) len=strlen(buffer);
TxLen=len;

/* The TX ISR is turned off in the normal state.*/
sim.uarts[port].uisr = 0x03; /* Enable the TX ISR */
sim.uarts[port].ucr = 4; /* Tx enable */ 
	
/* Wait until the tranmitter is done */
OSSemPend(&SemDataTx,0);
}






void UserMain(void * pd)
{
   InitializeStack();
   if (EthernetIP == 0) GetDHCPAddress(); 
   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();

   OpenBareSerial(1,115200);

   iprintf("Application started\n");

   IsrDrivenWrite(1,"Starting Remember to terminate your input with a \\n \r\n",0);
   while (1)
   {  int put_index,get_index;

	   OSSemPend(&SemDataRx,0);
	   /* We don't want to be interrupted while we get the indexes */
	   USER_ENTER_CRITICAL();
	   put_index=char_put_index; 
	   get_index=char_get_index; 
	   USER_EXIT_CRITICAL();
	   

	   /* Now to print out our circular buffer */
	   iprintf("We got a buffer[");
	   {
	   while (get_index!=put_index)
		{
		 iprintf("%c",RxBuffer[get_index++]);
		 if (get_index==1000) get_index=0;
		}
	   char_get_index=get_index; 
	   iprintf("]\r\n");
	   IsrDrivenWrite(1,"Done\r\n",0);
	   }
  }
}
